@@ -545,7 +545,7 @@ def plot_tbyt_decoding_acc(acc, start_time=0, end_time=1, time_interval=0.01, ch
 
     for t in range(nts):
         if ps[t] == 1:
-            plt.plot(t*tstep+start_time, (ymaxlim-yminlim)*0.9+yminlim, 's', color=color, alpha=0.8)
+            plt.plot(t*tstep+start_time+0.5*tstep, (ymaxlim-yminlim)*0.9+yminlim, 's', color=color, alpha=0.8)
             xi = [t*tstep+start_time, t*tstep+tstep+start_time]
             ymin = [chance]
             ymax = [avg[t] - err[t]]
